/*******************************************************************************
 * Copyright (c) 2007 Innovent Solutions, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Innovent Solutions, Inc.  - initial API and implementation
 *  Anthony Ku Ong - anthonykuong@gmail.com
 *******************************************************************************/

package org.eclipse.datatools.connectivity.oda.google.ui.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.datatools.connectivity.oda.IConnection;
import org.eclipse.datatools.connectivity.oda.IDriver;
import org.eclipse.datatools.connectivity.oda.IParameterMetaData;
import org.eclipse.datatools.connectivity.oda.IQuery;
import org.eclipse.datatools.connectivity.oda.IResultSetMetaData;
import org.eclipse.datatools.connectivity.oda.OdaException;
import org.eclipse.datatools.connectivity.oda.design.DataSetDesign;
import org.eclipse.datatools.connectivity.oda.design.DataSetParameters;
import org.eclipse.datatools.connectivity.oda.design.DesignFactory;
import org.eclipse.datatools.connectivity.oda.design.ParameterDefinition;
import org.eclipse.datatools.connectivity.oda.design.Properties;
import org.eclipse.datatools.connectivity.oda.design.ResultSetColumns;
import org.eclipse.datatools.connectivity.oda.design.ResultSetDefinition;
import org.eclipse.datatools.connectivity.oda.design.ui.designsession.DesignSessionUtil;
import org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage;
import org.eclipse.datatools.connectivity.oda.design.util.DesignUtil;
import org.eclipse.datatools.connectivity.oda.google.impl.GoogleConnection;
import org.eclipse.datatools.connectivity.oda.google.impl.GoogleDriver;
import org.eclipse.datatools.connectivity.oda.google.ui.model.GoogleModel;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * Auto-generated implementation of an ODA data set designer page
 * for an user to create or edit an ODA data set design instance.
 * This custom page provides a simple Query Text control for user input.  
 * It further extends the DTP design-time framework to update
 * an ODA data set design instance based on the query's derived meta-data.
 * <br>
 * A custom ODA designer is expected to change this exemplary implementation 
 * as appropriate. 
 */
public class CustomDataSetWizardPage extends DataSetWizardPage
{

    private static String DEFAULT_MESSAGE = "Define the query text for the data set";
    private transient GoogleModel model;
    private transient Text m_queryTextField;
    private transient Combo mSpreadSheetsCombo;
    private transient Combo mWorksheetsCombo;
    private boolean enableTestMode = true;

	/**
     * Constructor
	 * @param pageName
	 */
	public CustomDataSetWizardPage( String pageName )
	{
        super( pageName );
        setTitle( pageName );
        setMessage( DEFAULT_MESSAGE );

        setPageComplete( false );
	}

	/**
     * Constructor
	 * @param pageName
	 * @param title
	 * @param titleImage
	 */
	public CustomDataSetWizardPage( String pageName, String title,
			ImageDescriptor titleImage )
	{
        super( pageName, title, titleImage );
        setMessage( DEFAULT_MESSAGE );
        setPageComplete( false );
	}

	/* (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage#createPageCustomControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPageCustomControl( Composite parent )
	{
        setControl( createPageControl( parent ) );
        initializeControl();
	}
          
    /**
     * Creates custom control for user-defined query text.
     */
    public Control createPageControl( Composite parent )
    {
    	initModel();

    	Composite composite = new Composite( parent, SWT.NONE );
        composite.setLayout( new GridLayout( 1, false ) );
        
        GridData gridData = new GridData(GridData.FILL_BOTH);
		//gridData.widthHint = 450;
		GridLayout paneLayout = new GridLayout();
		paneLayout.marginHeight = paneLayout.marginWidth = 0;
		//Spreadsheet Combo
		String firstSpreadSheet = createSpreadSheetCombo(composite);
		//worksheetCombo
		String firstWorkSheet = createWorkSheetCombo(composite,firstSpreadSheet);
		// Query
		createTextualQueryComposite(composite);
		updateUrlTextField(firstSpreadSheet,firstWorkSheet);
		
		return composite;
    }
    
    
	private String createSpreadSheetCombo(Composite parent) {
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		Label label = new Label(parent, SWT.LEFT);
		label.setText("Available Spreadsheets :");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.minimumWidth = 150;
		mSpreadSheetsCombo = new Combo(parent, SWT.DROP_DOWN | SWT.READ_ONLY);
		
		ArrayList spreadSheetList = new ArrayList();
		Iterator it =  model.getSpreadSheet_WorkSheet().entrySet().iterator();
		    while (it.hasNext()) {
		        Map.Entry pairs = (Map.Entry)it.next();
		        String thisSpreadSheet = (String) pairs.getKey();
		        spreadSheetList.add(thisSpreadSheet);
		 }
		mSpreadSheetsCombo.setItems((String[])spreadSheetList.toArray(new String[spreadSheetList.size()]));
		mSpreadSheetsCombo.setLayoutData(gridData);
		// Sets current worksheet
		if (spreadSheetList.size() > 0) {
			int selectedIndex = 0;
			mSpreadSheetsCombo.select(selectedIndex);	
		}
		mSpreadSheetsCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				String selSpreadSheet = mSpreadSheetsCombo.getText();
				updateWorkSheetCombo(selSpreadSheet);
			}
		});
		if (mSpreadSheetsCombo.getItemCount() != 0) {
			return mSpreadSheetsCombo.getItem(0);
		} else {
			return null;
		}
	}
	
    
	private String createWorkSheetCombo(Composite parent, String firstSpreadSheet) {
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		Label label = new Label(parent, SWT.LEFT);
		label.setText("Available Worksheets :");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.minimumWidth = 150;
		mWorksheetsCombo = new Combo(parent, SWT.DROP_DOWN | SWT.READ_ONLY);
		mWorksheetsCombo.setItems(model.getWorkSheetIds());
		mWorksheetsCombo.setLayoutData(gridData);
		// Sets current worksheet
		if (firstSpreadSheet != null ){
			updateWorkSheetCombo(firstSpreadSheet);
		}
		mWorksheetsCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				updateUrlFeed();
		}});
		if (model.getWorkSheetIds().length > 0) {
			return mWorksheetsCombo.getItem(0);
		} else {
			return null;
		}
		
	}
	
	private void updateWorkSheetCombo(String selSpreadSheet){
		ArrayList workSheetList = (ArrayList) model.getSpreadSheet_WorkSheet().get(selSpreadSheet);
		if (workSheetList != null){
			mWorksheetsCombo.setItems((String[])workSheetList.toArray(new String[workSheetList.size()]));
			if (workSheetList.size() > 0) {
				int selectedIndex = 0;
				mWorksheetsCombo.select(selectedIndex);	
			}
		}
		updateUrlFeed();
	}
	
	/**
	 * Updates URL feed based on the currently selected Spreadsheet and Worksheet
	 */
	private void updateUrlFeed(){
		
		if ( mSpreadSheetsCombo.getText()!= null && mWorksheetsCombo.getText() != null) {
			String selSpreadSheet = mSpreadSheetsCombo.getText();
			String selWorkSheet = mWorksheetsCombo.getText();
			updateUrlTextField(selSpreadSheet, selWorkSheet);
		}
		
	}
	
	private void updateUrlTextField(String selSpreadSheet, String selWorkSheet){
		if (selSpreadSheet!= null && selWorkSheet != null) {
			String spreadSheetWorkSheet = selSpreadSheet + "." + selWorkSheet;
			HashMap sheetMap = model.getSpreadSheetModelMap();
			java.net.URL updateQueryURL = (java.net.URL) sheetMap.get(spreadSheetWorkSheet);
			String updateQuery = (updateQueryURL != null) ? updateQueryURL.toString(): null;
			if (updateQuery != null && m_queryTextField!= null){
				m_queryTextField.setText(updateQuery);
			}
		}
	}
    
    /**
	 * Creates the textual query editor
	 * 
	 * @param parent
	 */
	private void createTextualQueryComposite(Composite parent) {
		Composite composite = new Composite(parent, SWT.FILL
				| SWT.LEFT_TO_RIGHT);

		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		composite.setLayout(layout);

		Label fieldLabel = new Label( composite, SWT.NONE );
        fieldLabel.setText( "&Feed URL:" );
        m_queryTextField = new Text( composite, SWT.BORDER);
        GridData data = new GridData( GridData.FILL_HORIZONTAL );
       //data.widthHint = 450;
        m_queryTextField.setLayoutData( data );
        m_queryTextField.addModifyListener( new ModifyListener( ) 
        {
          public void modifyText( ModifyEvent e )
         {
            validateData();
         }}
         );
        
        if (!enableTestMode){
        	fieldLabel.setVisible(false);
        	m_queryTextField.setVisible(false);
        }
	
	}
   
	// ################## initialization ##############
	private void initModel() {
		if (model != null) {
			return;
		}
		String user = null;
		String pass = null;
		if (getInitializationDesign() != null) {
		    java.util.Properties connProps = 
                DesignUtil.convertDataSourceProperties( 
                        getInitializationDesign().getDataSourceDesign() );
			model = new GoogleModel();
			model.setProperties(connProps);
			model.initialize();
		}
		
	
	}
	/**
	 * Used for UI Testing 
	 * 
	 * */
	public void setModel(GoogleModel googleModel) {
		model = googleModel;
	}
	
	public void enableTestMode(boolean testMode){
		enableTestMode = testMode;
	}

	/**
	 * Initializes the page control with the last edited data set design.
	 */
	private void initializeControl( )
	{
        /* 
         * To optionally restore the designer state of the previous design session, use
         *      getInitializationDesignerState(); 
         */

        // Restores the last saved data set design
        DataSetDesign dataSetDesign = getInitializationDesign();
        if( dataSetDesign == null )
            return; // nothing to initialize

        String queryText = dataSetDesign.getQueryText();
        if( queryText == null || queryText.equals("") ){
         	updateUrlFeed();
        	return; 
        
        }   
        // initialize control from existing query
        m_queryTextField.setText( queryText );
        validateData();
        setMessage( DEFAULT_MESSAGE );

        /*
         * To optionally honor the request for an editable or
         * read-only design session, use
         *      isSessionEditable();
         */
	}
	
    /**
     * Obtains the user-defined query text of this data set from page control.
     * @return query text
     */
    private String getQueryText( )
    {
        return m_queryTextField.getText();
    }

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage#collectDataSetDesign(org.eclipse.datatools.connectivity.oda.design.DataSetDesign)
	 */
	protected DataSetDesign collectDataSetDesign( DataSetDesign design )
	{
        if( ! hasValidData() )
            return design;
        savePage( design );
        return design;
	}

    /*
     * (non-Javadoc)
     * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage#collectResponseState()
     */
	protected void collectResponseState( )
	{
		super.collectResponseState( );
		/*
		 * To optionally assign a custom response state, for inclusion in the ODA
		 * design session response, use 
         *      setResponseSessionStatus( SessionStatus status );
         *      setResponseDesignerState( DesignerState customState );
		 */
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.datatools.connectivity.oda.design.ui.wizards.DataSetWizardPage#canLeave()
	 */
	protected boolean canLeave( )
	{
        return isPageComplete();
	}

    /**
     * Validates the user-defined value in the page control exists
     * and not a blank text.
     * Set page message accordingly.
     */
	private void validateData( )
	{
        boolean isValid = ( m_queryTextField != null &&
            getQueryText() != null && getQueryText().trim().length() > 0 );

        if( isValid )
            setMessage( DEFAULT_MESSAGE );
        else
            setMessage( "Requires input value.", ERROR );

		setPageComplete( isValid );
	}

	/**
	 * Indicates whether the custom page has valid data to proceed 
     * with defining a data set.
	 */
	private boolean hasValidData( )
	{
        validateData( );
        
		return canLeave();
	}

	/**
     * Saves the user-defined value in this page, and updates the specified 
     * dataSetDesign with the latest design definition.
	 */
	private void savePage( DataSetDesign dataSetDesign )
	{
        // save user-defined query text
        String queryText = getQueryText();
        dataSetDesign.setQueryText( queryText );

        // obtain query's current runtime metadata, and maps it to the dataSetDesign
        IConnection customConn = null;
        try
        {
            // instantiate your custom ODA runtime driver class
            /* Note: You may need to manually update your ODA runtime extension's
             * plug-in manifest to export its package for visibility here.
             */
            IDriver customDriver = new GoogleDriver();
            
            // obtain and open a live connection
            customConn = customDriver.getConnection( null );
            java.util.Properties connProps = 
                DesignUtil.convertDataSourceProperties( 
                        getInitializationDesign().getDataSourceDesign() );
            customConn.open( connProps );

            // update the data set design with the 
            // query's current runtime metadata
            updateDesign( dataSetDesign, customConn, queryText );
        }
        catch( OdaException e )
        {
            // not able to get current metadata, reset previous derived metadata
            dataSetDesign.setResultSets( null );
            dataSetDesign.setParameters( null );
            
            e.printStackTrace();
        }
        finally
        {
            closeConnection( customConn );
        }
	}

    /**
     * Updates the given dataSetDesign with the queryText and its derived metadata
     * obtained from the ODA runtime connection.
     */
    private void updateDesign( DataSetDesign dataSetDesign,
                               IConnection conn, String queryText )
        throws OdaException
    {
        IQuery query = conn.newQuery( null );
        query.prepare( queryText );
        
        // TODO a runtime driver might require a query to first execute before
        // its metadata is available
//      query.setMaxRows( 1 );
//      query.executeQuery();
        
        try
        {
            IResultSetMetaData md = query.getMetaData();
            updateResultSetDesign( md, dataSetDesign );
        }
        catch( OdaException e )
        {
            // no result set definition available, reset previous derived metadata
            dataSetDesign.setResultSets( null );
            e.printStackTrace();
        }
        
        // proceed to get parameter design definition
        try
        {
            IParameterMetaData paramMd = query.getParameterMetaData();
            updateParameterDesign( paramMd, dataSetDesign );
        }
        catch( OdaException ex )
        {
            // no parameter definition available, reset previous derived metadata
            dataSetDesign.setParameters( null );
            ex.printStackTrace();
        }
        
        /*
         * See DesignSessionUtil for more convenience methods
         * to define a data set design instance.  
         */     
    }

    /**
     * Updates the specified data set design's result set definition based on the
     * specified runtime metadata.
     * @param md    runtime result set metadata instance
     * @param dataSetDesign     data set design instance to update
     * @throws OdaException
     */
	private void updateResultSetDesign( IResultSetMetaData md,
            DataSetDesign dataSetDesign ) 
        throws OdaException
	{
        ResultSetColumns columns = DesignSessionUtil.toResultSetColumnsDesign( md );

        ResultSetDefinition resultSetDefn = DesignFactory.eINSTANCE
                .createResultSetDefinition();
        // resultSetDefn.setName( value );  // result set name
        resultSetDefn.setResultSetColumns( columns );

        // no exception in conversion; go ahead and assign to specified dataSetDesign
        dataSetDesign.setPrimaryResultSet( resultSetDefn );
        dataSetDesign.getResultSets().setDerivedMetaData( true );
	}

    /**
     * Updates the specified data set design's parameter definition based on the
     * specified runtime metadata.
     * @param paramMd   runtime parameter metadata instance
     * @param dataSetDesign     data set design instance to update
     * @throws OdaException
     */
    private void updateParameterDesign( IParameterMetaData paramMd,
            DataSetDesign dataSetDesign ) 
        throws OdaException
    {
        DataSetParameters paramDesign = 
            DesignSessionUtil.toDataSetParametersDesign( paramMd, 
                    DesignSessionUtil.toParameterModeDesign( IParameterMetaData.parameterModeIn ) );
        
        // no exception in conversion; go ahead and assign to specified dataSetDesign
        paramDesign.setDerivedMetaData( true );
        dataSetDesign.setParameters( paramDesign );

        // TODO replace below with data source specific implementation;
        // hard-coded parameter's default value for demo purpose
        if( paramDesign.getParameterDefinitions().size() > 0 )
        {
            ParameterDefinition paramDef = 
                (ParameterDefinition) paramDesign.getParameterDefinitions().get( 0 );
            if( paramDef != null )
                paramDef.setDefaultScalarValue( "dummy default value" );
        }
    }

    /**
     * Attempts to close given ODA connection.
     */
    private void closeConnection( IConnection conn )
    {
        try
        {
            if( conn != null && conn.isOpen() )
                conn.close();
        }
        catch ( OdaException e )
        {
            // ignore
            e.printStackTrace();
        }
    }

}
