/*******************************************************************************
 * Copyright (c) 2008 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Anthony Ku Ong   - initial API and implementation
 *******************************************************************************/
package org.eclipse.datatools.connectivity.oda.google.ui.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import org.eclipse.datatools.connectivity.oda.OdaException;
import org.eclipse.datatools.connectivity.oda.design.util.DesignUtil;
import org.eclipse.datatools.connectivity.oda.google.impl.GoogleConnection;
import org.eclipse.datatools.connectivity.oda.google.impl.GoogleDriver;
import org.eclipse.datatools.connectivity.oda.google.impl.GoogleQuery;
import org.eclipse.datatools.connectivity.oda.google.impl.GoogleSpreadsheetModel;

public class GoogleModel {
	private GoogleConnection googleConnection;
	private GoogleDriver googleDriver;
	private GoogleQuery googleQuery;
	private HashMap googleSpreadSheetModelMap;

	private HashMap spreadSheet_workSheet;

	private ArrayList workSheetIds;
	private ArrayList feedUrls;
	private String username;
	private String password;
	private Properties connProps;
	private GoogleSpreadsheetModel googleSpreadSheetModel;

	public GoogleModel() {
		spreadSheet_workSheet = new HashMap();
		workSheetIds = new ArrayList();
		feedUrls = new ArrayList();
	}

	public HashMap getSpreadSheet_WorkSheet() {
		return spreadSheet_workSheet;
	}

	public String[] getWorkSheetIds() {
		return (String[]) workSheetIds.toArray(new String[workSheetIds.size()]);
	}

	public HashMap getSpreadSheetModelMap() {
		return googleSpreadSheetModelMap;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public void setProperties(Properties connProps) {
		this.connProps = connProps;
		setUsername(connProps.getProperty("username"));
		setPassword(connProps.getProperty("password"));
	}

	public void initialize() {
		// obtain query's current runtime metadata, and maps it to the
		// dataSetDesign
		if (connProps == null) {
			throw new NullPointerException();
		}
		googleConnection = null;
		try {
			googleDriver = new GoogleDriver();
			googleConnection = (GoogleConnection) googleDriver
					.getConnection(null);
			googleConnection.open(connProps);
		} catch (OdaException e) {
			e.printStackTrace();
		}

		try {
			googleQuery = (GoogleQuery) googleConnection.newQuery(null);
		} catch (OdaException e) {
			e.printStackTrace();
		}
		createGoogleSpreadSheetModelUI();
	}

	private void createGoogleSpreadSheetModelUI() {
		googleSpreadSheetModel = googleQuery.getGoogleSpreadSheetModelObject();

		if (googleSpreadSheetModel == null) {
			throw new NullPointerException("Empty Spreadsheet Model");
		} else {
			// Map that contains (SpreadsheetEntry, WorksheetEntries)
			spreadSheet_workSheet = googleSpreadSheetModel
					.getSpreadSheet_workSheet();
			// Map that contains (Spreadsheet.Worksheet, FeedURL )
			googleSpreadSheetModelMap = googleSpreadSheetModel
					.getSpreadSheetMap();
			Iterator it = googleSpreadSheetModelMap.entrySet().iterator();

			while (it.hasNext()) {
				Map.Entry pairs = (Map.Entry) it.next();
				workSheetIds.add(pairs.getKey());
				feedUrls.add(pairs.getValue());
			}
		}
	}

	/** Tests
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		Properties connProps = new Properties();
		connProps.put("username", "birt.spreadsheet");
		connProps.put("password", "birtexchange");
		// connProps.put("username", "info@innoventsolutions.com");
		// connProps.put("password", "innovent2007" );
		GoogleModel googleModel = new GoogleModel();
		googleModel.setProperties(connProps);
		googleModel.initialize();

	}

}
