/*******************************************************************************
 * Copyright (c) 2008 
 * 
 * Creates a GoogleSpreadsheetModel from the SpreadsheetService 
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Anthony Ku Ong   - initial API and implementation
 *******************************************************************************/

package org.eclipse.datatools.connectivity.oda.google.impl;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Logger;

import org.eclipse.datatools.connectivity.oda.OdaException;

import com.google.gdata.client.spreadsheet.FeedURLFactory;
import com.google.gdata.client.spreadsheet.ListQuery;
import com.google.gdata.client.spreadsheet.SpreadsheetService;
import com.google.gdata.data.spreadsheet.ListEntry;
import com.google.gdata.data.spreadsheet.ListFeed;
import com.google.gdata.data.spreadsheet.SpreadsheetEntry;
import com.google.gdata.data.spreadsheet.SpreadsheetFeed;
import com.google.gdata.data.spreadsheet.WorksheetEntry;
import com.google.gdata.data.spreadsheet.WorksheetFeed;
import com.google.gdata.util.ServiceException;
public class GoogleSpreadsheetModel {
	
	private HashMap mSpreadSheetMap;
	private SpreadsheetService service;
	private SpreadsheetFeed feed;
	private HashMap spreadSheet_workSheet;
	private ArrayList spreadSheet_names;
	private ArrayList workSheet_feedUrls;
	protected Logger logger = Logger.getLogger(GoogleSpreadsheetModel.class.getName());
	/**
	 * 
	 */
	public GoogleSpreadsheetModel() {
	}

	public GoogleSpreadsheetModel(SpreadsheetService service) {
		this.service = service;
		try {
			createFeed();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();
		}
		initModel();
	}
	
	public HashMap getSpreadSheet_workSheet() {
		return spreadSheet_workSheet; 
	}
	
	public HashMap getSpreadSheetMap(){
		return mSpreadSheetMap;
	}
	
	public ArrayList getSpreadSheetNames(){
		return spreadSheet_names;
	}
	

	public ArrayList getWorkSheetURLs() {
		return workSheet_feedUrls;
	}
	
	
	private void createFeed() throws IOException, ServiceException{
		FeedURLFactory factory = FeedURLFactory.getDefault();
		feed = service.getFeed(factory.getSpreadsheetsFeedUrl(), SpreadsheetFeed.class);
		
		
		
	}

	@SuppressWarnings("unchecked")
	public void initModel() {
	    spreadSheet_workSheet = new HashMap();
		mSpreadSheetMap = new HashMap();
		spreadSheet_names = new ArrayList();
		workSheet_feedUrls = new ArrayList();
		
		if (feed == null || feed.getEntries() == null || feed.getEntries().size() == 0) {
			logger.severe("Empty or null Feed");
			return;
		}
		// Iterate over each SpreadSheet, get the worksheet feed.
		if (feed.getEntries().isEmpty()){
			logger.severe("No spreadsheets");
			return;
		}
		for (SpreadsheetEntry spreadsheet : feed.getEntries())
		{
		
			String spreadSheetName  = spreadsheet.getTitle().getPlainText();
			String workSheetName = "";
			
			//System.out.println(spreadsheet.getTitle().getPlainText() + " : " + spreadsheet.getCanEdit() + ":"+ spreadsheet.getHtmlLink().getHref() + ":" + spreadsheet.getUpdated().toUiString());
			WorksheetFeed worksheet_feed = null;
			try {
				worksheet_feed = service.getFeed(spreadsheet.getWorksheetFeedUrl(), WorksheetFeed.class);
			} catch (IOException e) {
				e.printStackTrace();
			} catch (ServiceException e) {
				logger.severe("Problem getting Worksheet Feed:" + e.getMessage());
				continue;
			}
			
			ArrayList currWorkSheet_names = new ArrayList();
			for (WorksheetEntry worksheet : worksheet_feed.getEntries()) {
				logger.fine(" -" + worksheet.getTitle().getPlainText() + " " + worksheet.getListFeedUrl());
				workSheetName = worksheet.getTitle().getPlainText();
				spreadSheet_names.add(spreadSheetName);
				currWorkSheet_names.add(workSheetName);
					workSheet_feedUrls.add(worksheet.getListFeedUrl());
				mSpreadSheetMap.put(spreadSheetName + "." + workSheetName, worksheet.getListFeedUrl());
			}
			spreadSheet_workSheet.put(spreadSheetName,currWorkSheet_names);
		}
		
	}
	
	public void showWorkSheet(SpreadsheetService service, URL feedUrl) throws IOException, ServiceException {
		ListQuery query = new ListQuery(feedUrl);
		
		// Use the ListFeed to work with the entire worksheet contents
		ListFeed feed = service.query(query, ListFeed.class);
		if (feed.getEntries().isEmpty()){
			logger.severe("Not a valid worksheet!");
		}
		
		for (ListEntry listEntry : feed.getEntries()) {
			for (String tag : listEntry.getCustomElements().getTags()) {
				String rowName = listEntry.getTitle().getPlainText();
				String colName = tag;
				String cellValue =listEntry.getCustomElements().getValue(tag); 
				logger.finest("row(" + rowName + ") col( " + colName + ") "	+ cellValue );
			}
		}
	}
}
